/*
 * vopstat - Trace the vnode interface.
 *           Written using DTrace (Solaris 10 3/05)
 *
 * Author: Richard McDougall
 *
 * $Id: vopstat 3 2007-08-01 10:50:08Z brendan $
 *
 * COPYRIGHT: Copyright (c) 2006 Richard McDougall
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * Shell Wrapper Concept by Brendan Gregg
 *
 * 08-Jan-2006	Richard McDougall	Created this.
 * 23-Apr-2006	Brendan Gregg		Minor style tweaks.
 * 23-Apr-2006	   "      "		Last update.
 * 06-Nov-2008	Tom Erickson		(adapted for Chime)
 */

/*
 * Copyright 2009 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

inline int OPT_all 	= ??ALL??;
inline string FILESYS 	= "??FILESYS??";
 
dtrace:::BEGIN 
{
	self->path = "";
	self->trace = 0;
}

fbt::fop_*:entry
{
 	self->trace = 0;

 	/* Get vp: fop_open has a pointer to vp */
 	this->vpp = (vnode_t **)arg0;
 	self->vp = (vnode_t *)arg0;
 	self->vp = probefunc == "fop_open" ? (vnode_t *)*this->vpp : self->vp;

 	/* And the containing vfs */
         this->vfsp = self->vp ? self->vp->v_vfsp : 0;

 	/* And the paths for the vp and containing vfs */
 	this->vfsvp = this->vfsp ?
	    (struct vnode *)((vfs_t *)this->vfsp)->vfs_vnodecovered : 0;
 	self->vfspath = this->vfsvp ? stringof(this->vfsvp->v_path) : "unknown";
 
 	/* Check if we should trace the root fs */
 	(OPT_all || 
 	    (FILESYS == "/" && this->vfsp &&
 	    (this->vfsp == `rootvfs))) ? self->trace = 1 : self->trace;
 
 	/* Check if we should trace the fs */
 	(OPT_all || (self->vfspath == FILESYS)) ? self->trace = 1 : self->trace;

	self->vfspath = 0;
}
 
/*
 * Trace the entry point to each fop
 */ 
fbt::fop_*:entry
/self->trace/
{
 	self->path = (self->vp != NULL && self->vp->v_path) ?
	    stringof(self->vp->v_path) : "unknown";
 
 	/* Some fops has the len in arg2 */
 	(probefunc == "fop_getpage" ||
 	 probefunc == "fop_putpage" ||
 	 probefunc == "fop_none") ? self->len = arg2 : 1; 
 
 	/* Some fops has the len in arg3 */
 	(probefunc == "fop_pageio" ||
 	 probefunc == "fop_none") ? self->len = arg3 : 1; 
 
 	/* Some fops has the len in arg4 */
 	(probefunc == "fop_addmap" ||
 	 probefunc == "fop_map" ||
 	 probefunc == "fop_delmap") ? self->len = arg4 : 1; 
 
 	/* Some fops has the offset in arg1 */
 	(probefunc == "fop_addmap" ||
 	 probefunc == "fop_map" ||
 	 probefunc == "fop_getpage" ||
 	 probefunc == "fop_putpage" ||
 	 probefunc == "fop_seek" ||
 	 probefunc == "fop_delmap") ? self->off = arg1 : 1; 
 
 	/* Some fops has the offset in arg3 */
 	(probefunc == "fop_close" ||
 	 probefunc == "fop_pageio") ? self->off = arg3 : 1; 
 
 	/* Some fops has the offset in arg4 */
 	probefunc == "fop_frlock" ? self->off = arg4 : 1; 
 
 	/* Some fops has the pathname in arg1 */
 	self->path = (probefunc == "fop_create" ||
 	 probefunc == "fop_mkdir" ||
 	 probefunc == "fop_rmdir" ||
 	 probefunc == "fop_remove" ||
 	 probefunc == "fop_lookup") ?
 	    strjoin(self->path, strjoin("/", stringof(arg1))) : self->path;
 
 	self->type = probefunc;
 	self->vop_entry[probefunc] = timestamp;
}
 
fbt::fop_*:return
/self->trace/
{
	@vop_time[probefunc] = sum(timestamp - self->vop_entry[probefunc]);
	@vop_cnt[probefunc] = count();

	self->path = 0;
	self->len = 0;
	self->off = 0;
	self->vop_entry[probefunc] = 0;
}

fbt::fop_*:return
{
	self->trace = 0;
	self->type = 0;
	self->vp = 0;
}
 
/* Capture any I/O within this fop */
io:::start
/self->trace/
{
	@vop_iocnt[self->type] = count();
}

??ALL 1
??FILESYS
